#!/bin/bash

APACHE_CONFDIR="/etc/httpd2/conf"

#turn off auto expansion
set -f

print_aes_name()
{
	local object="$1"
	local name="$2"

	case "$name" in
		available)
			printf '%s-available' "$object"
			;;
		enabled)
			printf '%s-enabled' "$object"
			;;
		start)
			printf '%s-start.d' "$object"
			;;
		*)
			printf ''
			;;
	esac
}

print_a2used_name()
{
	local object="${1%s}"
	local enabled="$2"
	local s=

	case "$enabled" in
		yes)
			s="en"
			;;
		no)
			s="dis"
			;;
	esac
	printf 'a2%s%s' "$s" "$object"
}

print_dir_requires()
{
	printf '("Directory" label "%s")\n' "`_ "any directory"`"
	printf '("Directory-root" label "%s")\n' "`_ "root directory"`"
	printf '("Directory-html" label "%s")\n' "`_ "html directory"`"
	printf '("Directory-cgi" label "%s")\n' "`_ "CGI directory"`"
}

print_dir_actions()
{
	printf '("nop" label "-- %s --")\n' "`_ "select action"`"
	printf '("reset_dir" label "%s")\n' "`_ "reset directory"`"
	printf '("delete" label "%s")\n' "`_ "delete"`"
}

print_alias_actions()
{
	printf '("nop" label "-- %s --")\n' "`_ "select action"`"
	printf '("reset_alias" label "%s")\n' "`_ "reset alias"`"
	printf '("delete" label "%s")\n' "`_ "delete"`"
}

print_avail_actions()
{
	printf '("nop" label "-- %s --")\n' "`_ "select action"`"
	printf '("none_def" label "%s")\n' "`_ "default none"`"
	printf '("en_def" label "%s")\n' "`_ "enable by default"`"
	printf '("dis_def" label "%s")\n' "`_ "disable by default"`"
	printf '("enable" label "%s")\n' "`_ "enable"`"
	printf '("disable" label "%s")\n' "`_ "disable"`"
	printf '("delete" label "%s")\n' "`_ "delete"`"
}

P_sarg="[^[:space:]>\'\"]"
P_parg="[[:space:]>\'\"]"
P_arg="$P_parg*$P_sarg+$P_parg*"
P_send="[[:space:]]*(|([[:space:]]*#.*))\$"

P_VirtualHost="[Vv][Ii][Rr][Tt][Uu][Aa][Ll][Hh][Oo][Ss][Tt]"
P_Directory="[Dd][Ii][Rr][Ee][Cc][Tt][Oo][Rr][Yy]"
P_IfModule="[Ii][Ff][Mm][Oo][Dd][Uu][Ll][Ee]"

# P_{{F,}O,Z}T_* generate
for p in P_VirtualHost P_Directory P_IfModule;
do
	eval "P_FOT_${p#P_}=\"^[[:space:]]*<\$$p[[:space:]]\$P_parg*\""
	eval "P_OT_${p#P_}=\"\$P_FOT_${p#P_}\$P_arg>\$P_send\""
	eval "P_ZT_${p#P_}=\"^[[:space:]]*<\/\$$p>\$P_send\""
done

P_pre_VirtualHost="1,/$P_OT_VirtualHost/{b}
/$P_ZT_VirtualHost/,\${b}
"
P_post_VirtualHost="
"


P_Server="[Ss][Ee][Rr][Vv][Ee][Rr]"
P_Name="[Nn][Aa][Mm][Ee]"
P_Path="[Pp][Aa][Tt][Hh]"
P_Admin="[Aa][Dd][Mm][Ii][Nn]"
P_ServerName="$P_Server$P_Name"
P_ServerPath="$P_Server$P_Path"
P_ServerAdmin="$P_Server$P_Admin"
P_DocumentRoot="[Dd][Oo][Cc][Uu][Mm][Ee][Nn][Tt][Rr][Oo][Oo][Tt]"
P_ErrorLog="[Ee][Rr][Rr][Oo][Rr][Ll][Oo][Gg]"
P_Include="[Ii][Nn][Cc][Ll][Uu][Dd][Ee]"
P_Alias="[Aa][Ll][Ii][Aa][Ss]"
P_Script="[Ss][Cc][Rr][Ii][Pp][Tt]"
P_ScriptAlias="$P_Script$P_Alias"

# P_f{,s}_* generate
for p in P_ServerName P_ServerPath P_ServerAdmin \
	P_Alias P_ScriptAlias \
	P_DocumentRoot P_ErrorLog P_Include;
do
	eval "P_f_${p#P_}=\"\$$p[[:space:]]\""
	eval "P_fs_${p#P_}=\"^[[:space:]]*\$$p[[:space:]]\""
done

P_pre_Headers="1,/^[[:space:]]*[^[:space:]]/{
	/^[[:space:]]*[^[:space:]]/b e1
	b
}
: e1
/^[[:space:]]*$/,$ b
/^[[:space:]]*[^[:space:]#].*$/,$ b
"
P_post_Headers="
"
P_pre_Headers_Directory="
/$P_FOT_Directory/,1 b
/$P_ZT_Directory/,$ b
$P_pre_Headers"
P_post_Headers_Directory="$P_post_Headers"

P_Summary="Summary"
P_Provides="Provides"
P_Requires="Requires"

# P_{f{,s},pre,post}_* generate
for p in P_Summary P_Provides P_Requires;
do
	eval "P_f_${p#P_}=\"\#[[:space:]]*\$$p:\""
	eval "P_fs_${p#P_}=\"^[[:space:]]*\$P_f_${p#P_}\""
	eval "P_pre_${p#P_}=\"$P_pre_Headers
1,/\$P_fs_${p#P_}/{
\""
	eval "P_post_${p#P_}=\"$P_post_Headers
}\""
done

# P_get_* generate
P_kav1="\\\'"
P_kav2="\\\\\\\\\\\\\""
P_kavs="$P_kav1$P_kav2"
P_sk="[:space:]$P_kavs"
P_fv="\\\\\\\\\\"
P_v1="${P_fv}1"
P_v2="${P_fv}2"
P_h1v="[[:space:]]*((([$P_kavs])(.*)(${P_fv}3))|([^[:space:]].*[^[:space:]]))[[:space:]]*\$"
P_h2v="[[:space:]]*((([$P_kavs])(.*)${P_fv}3)|([^[:space:]].*[^[:space:]]))[[:space:]]+(([$P_kavs])(.*)${P_fv}7)|([^[:space:]].*[^[:space:]])[[:space:]]*\$"
while read e
do
	eval "$e"
done < <(
	for p in P_Summary P_Provides P_Requires P_Include;
	do
		N="${p#P_}"
		printf '%s\n' \
			"P_get_$N=\"/\$P_fs_$N/s/\$P_fs_$N$P_h1v/$N\\\t$P_kav2${P_fv}4${P_fv}6$P_kav2/p\""
	done
	for p in P_Alias P_ScriptAlias;
	do
		N="${p#P_}"
		printf '%s\n' \
			"P_get_$N=\"/\$P_fs_$N/s/\$P_fs_$N$P_h2v/$N\\\t$P_kav2${P_fv}4${P_fv}5$P_kav2\\\t$P_kav2${P_fv}8${P_fv}9$P_kav2/p\""
	done
)

strip_kav()
{
	echo "$@" | sed -r "s@^[[:space:]]*((([\'\"])(.*)(\3))|([^[:space:]].*[^[:space:]]))[[:space:]]*\$@\4\6@"
}

get_headers()
{
	local fn="$1"
	sed -rn "${P_pre_Headers}
		$P_get_Summary
		$P_get_Provides
		$P_get_Requires
$P_post_Headers" \
		"$fn" | \
		sed -r "s/([$P_kavs])/\\\\\1/g" | \
		sed -r "s/^[[:space:]]*([^[:space:]]+)[[:space:]]+(\\\\([$P_kavs]))(.*)\2[[:space:]]*$/\1\t\3\4\3/"
}

get_summary()
{
	local fn="$1"
	get_headers "$@" | \
		sed -rn "/^$P_Summary[[:space:]]/s/^$P_Summary[[:space:]]+\"([^[:space:]].*[^[:space:]])\"[[:space:]]*\$/\1/p" | \
		head -n 1
}

get_dirs_headers()
{
	local n a
	local dir=""
	local md5=""
	sed -rn "$P_pre_VirtualHost
	/$P_OT_Directory/,/$P_ZT_Directory/{
		/$P_OT_Directory/s/[[:space:]]*<$P_Directory[[:space:]]$P_parg*($P_sarg+)$P_parg*>$P_send/Directory\t\1/p
		$P_get_Summary
		$P_get_Provides
		$P_get_Requires
		$P_get_Include
	}" \
		"$1" | \
		while read -r n a; do
			if [ "x$n" = "xDirectory" ]; then
				dir="$a"
				md5="`echo "$dir"|md5sum -t -|cut -d' ' -f 1`"
				printf 'md5\t%s\tDirectory\t%s\n' "$md5" "$dir"
				continue
			fi
			if [ "x$dir" != "x" ]; then
				if [ "x$n" != "xDirectory" ]; then
					printf 'md5\t%s\t%s\t%s\n' \
						"$md5" "$n" "$a"
				fi
			else
				continue
			fi
		done
}

get_IfModule()
{
	local fname="$1"
	sed -rn "$P_pre_VirtualHost
	/$P_OT_IfModule/s/[[:space:]]*<$P_IfModule[[:space:]]$P_parg*($P_sarg+)$P_parg*>$P_send/IfModule\t\1/p
$P_post_VirtualHost" \
			"$fname"
}

new_IfModule_alias()
{
	local fname="$1"
	sed -i -r "1,/$P_ZT_VirtualHost/{
	/$P_ZT_VirtualHost/{
		i \\\\t<IfModule alias_module>
		i \\\\t</IfModule>
	}
}" \
			"$fname"
}

get_alias()
{
	local n a d
	local alias=""
	local dir=""
	local md5=""
	sed -rn "$P_pre_VirtualHost
	$P_get_Alias
	$P_get_ScriptAlias
$P_post_VirtualHost" \
		"$1" | \
		while read -r n a d; do
			alias="`strip_kav "$a"`"
			dir="`strip_kav "$d"`"
			md5="`echo "$alias"|md5sum -t -|cut -d' ' -f 1`"
			printf 'md5\t%s\t%s\t\"%s\"\tDirectory\t\"%s\"\n' \
					"$md5" "$n" "$alias" "$dir"
		done
}

new_alias()
{
	local fname="$1"
	local type="$2"
	local alias="`strip_kav "$3"`"
	local dir="`strip_kav "$4"`"

	if [ "x$alias" != "x" ] &&
			[ "x$dir" != "x" ]; then

		get_IfModule "$fname" | \
			grep -q "^[[:space:]]*$P_IfModule[[:space:]]$P_parg*alias_module$P_parg*$" || \
			new_IfModule_alias "$fname"


		sed -i -r "$P_pre_VirtualHost
	1,/${P_FOT_IfModule}alias_module$P_parg*>$P_send/{
		/${P_FOT_IfModule}alias_module$P_parg*>$P_send/{
			a \\\\t\t$type $alias \"$dir\"
		}
	}
$P_post_VirtualHost" \
				"$fname"
	fi
}

set_Alias_opt()
{
	local fname="$1"
	local md5="$2"
	local type="$3"
	local alias="`strip_kav "$4"`"
	local dir="`strip_kav "$5"`"

	if [ "x$alias" != "x" ] &&
			[ "x$dir" != "x" ]; then
		get_alias "$fname" | \
			sed -nr "/^[[:space:]]*md5[[:space:]]+$md5[[:space:]]+$type[[:space:]]+[^[:space:]]+[[:space:]]+Directory[[:space:]]+[^[:space:]]+[[:space:]]*$/s/^[[:space:]]*md5[[:space:]]+$md5[[:space:]]+$type[[:space:]]+([^[:space:]]+)[[:space:]]+Directory[[:space:]]+[^[:space:]]+[[:space:]]*$/\1/p" | \
			{
				local a
				local oa=""
				local foa="no"

				while read -r a; do
					oa="`strip_kav "$a"`"
					foa="yes"
				done
				if [ "$foa" = "yes" ]; then
					local P_fs P

					eval "P_fs=\"\$P_fs$type\""
					eval "P=\"\$P_$type\""
					sed -i -r "$P_pre_VirtualHost
	\%$P_fs$P_parg*$oa$P_parg*%s%^([[:space:]]*)$P[[:space:]]*.*$%\1$type $alias \"$dir\"%
$P_post_VirtualHost" \
							"$fname"
				else
					new_alias "$fname" "$type" "$alias" "$dir"
				fi
			}
	else
		del_Alias "$fname" "$md5" "$type"
	fi
}

print_alias()
{
	local fname="$1"
	local pat="$2"
	get_alias "$fname" | \
		sed -nr "/^[[:space:]]*md5[[:space:]]+[^[:space:]]+[[:space:]]+$pat[[:space:]]+[^[:space:]]+[[:space:]]+Directory[[:space:]]+[^[:space:]]+[[:space:]]*$/s/^[[:space:]]*md5[[:space:]]+([^[:space:]]+)[[:space:]]+$pat[[:space:]]+([^[:space:]]+)[[:space:]]+Directory[[:space:]]+([^[:space:]]+)[[:space:]]*$/\(\"\1\" alias \2 dir \3\)/p"
}

dirs_md5_list()
{
	get_dirs_headers "$@" | \
		sed -rn '/^[[:space:]]*md5[[:space:]]+([^[:space:]]+)[[:space:]]+Directory[[:space:]]/s/^[[:space:]]*md5[[:space:]]+([^[:space:]]+)[[:space:]]+Directory[[:space:]]+([^[:space:]]+).*$/\1\t\2/p'
}

find_Provides_files()
{
	local req="`strip_kav "$1"`"

	find "$APACHE_CONFDIR/extra-available" -maxdepth 1 -name "*.conf" -type f -o -type l | \
		xargs grep -R "$P_fs_Provides[[:space:]]\+$req[[:space:]]*$" \
			2>/dev/null | \
		cut -d':' -f1 | \
		sort | \
		uniq | \
		{
			local fname=""
			while read -r fname; do
				get_headers "$fname" | \
					{
						local n a
						local summary=""
						local f="no"

						while read -r n a; do
							case "$n" in
								Summary)
									summary="$a"
									;;
								Provides)
									if [ "$a" = "$req" ]; then
										f="yes"
									fi
									;;
							esac
						done
						if [ "$f" = "yes" ]; then
							local inc
							inc="`echo $fname | sed -r "s@^$APACHE_CONFDIR/extra-available/([^\\'\\"]*)\.conf[[:space:]\\'\\"]*@\\1@"`"
							printf 'include "%s" summary "%s" fname "%s" label "%s -- %s"\n' \
								"$inc" "$summary" "$fname" "$inc" "$summary"
						fi
					}
			done
		}
}

new_Directory()
{
	local fname="$1"
	local dir="`strip_kav "$2"`"
	local req="`strip_kav "$3"`"
	local summary="`strip_kav "$4"`"
	local inc=""

	if [ "x$dir" = "x" ]; then
		return
	fi
	if [ "x$req" = "x" ]; then
		req="Directory"
	fi
	if [ "x$summary" = "x" ]; then
		summary="Directory, generated alterator-apache2"
	fi
	inc="`find_Provides_files "$req" | \
		head -n 1 | \
		sed -rn "/^[[:space:]]*include[[:space:]]/s/^[[:space:]]*include[[:space:]]+([^[:space:]]+)[[:space:]].*$/\1/p"`"
	inc="`strip_kav "$inc"`"
	if [ "x$inc" = "x" ]; then
		return
	fi
	inc="conf/extra-available/$inc.conf"

	sed -i -r "1,/$P_ZT_VirtualHost/{
	/$P_ZT_VirtualHost/{
		i \\\\t<Directory \"$dir\">
		i \\\\t\t\# Summary: $summary
		i \\\\t\t\# Requires: $req
		i \\\\t\tInclude $inc
		i \\\\t</Directory>
	}
}" \
			"$fname"
}

del_Directory()
{
	local fname="$1"
	local md5="`strip_kav "$2"`"

	local dir="`dirs_md5_list "$fname" | \
		sed -rn "/^[[:space:]]*$md5[[:space:]]/s/^[[:space:]]*$md5[[:space:]]+([^[:space:]]+)[[:space:]]*$/\1/p" | \
		head -n 1`"
	dir="`strip_kav "$dir"`"
	sed -i -r "$P_pre_VirtualHost
	\@[[:space:]]*<$P_Directory[[:space:]]$P_parg*($dir)$P_parg*>$P_send@,\@[[:space:]]*</$P_Directory>$P_send@d
$P_post_VirtualHost" \
		"$fname"
}

del_Alias()
{
	local fname="$1"
	local md5="`strip_kav "$2"`"
	local type="`strip_kav "$3"`"
	local P

	eval "P=\"\$P_$type\""

	local alias="`get_alias "$fname" | \
		sed -nr "/^[[:space:]]*md5[[:space:]]+$md5+[[:space:]]+$P[[:space:]]+[^[:space:]]+[[:space:]]+.*$/s/^[[:space:]]*md5[[:space:]]+$md5[[:space:]]+$P[[:space:]]+([^[:space:]]+)[[:space:]]+Directory[[:space:]]+.*$/\1/p" | \
		head -n 1`"
	alias="`strip_kav "$alias"`"
	sed -i -r "$P_pre_VirtualHost
	\%^[[:space:]]*$type[[:space:]]+$alias[[:space:]]+.*$% d
$P_post_VirtualHost" \
			"$fname"
}

set_Directory_opt()
{
	local fname="$1"
	local md5="`strip_kav "$2"`"
	local oname="`strip_kav "$3"`"
	local onew="`strip_kav "$4"`"

	get_dirs_headers "$fname" | \
		sed -rn "/^[[:space:]]*md5[[:space:]]+$md5[[:space:]]/s/^[[:space:]]*md5[[:space:]]+$md5[[:space:]]+([^[:space:]]+)[[:space:]]+(.+)[[:space:]]*$/\1\t\2/p" | \
		{
			local n a
			local dir=""
			local oold=""
			local fold="no"
			local fheader="no"

			while read -r n a; do
				n="`strip_kav "$n"`"
				if [ "$n" = "Directory" ]; then
					dir="`strip_kav "$a"`"
				elif [ "$n" = "$oname" ]; then
					oold="`strip_kav "$a"`"
					fold="yes"
				fi
				if [ "$n" = "Summary" ] ||
						[ "$n" = "Requires" ]; then
					fheader="yes"
				fi
			done
			if [ "x$dir" != "x" ]; then
				local P_pre="$P_pre_VirtualHost
\:$P_FOT_Directory$dir$P_parg*>$P_send:,/$P_ZT_Directory/{
	1,/$P_ZT_Directory/{"
				local P_post="	}
}
$P_post_VirtualHost"
				local P_pre2=""
				local P_post2=""
				local com=""
				local p_com

				eval "p_com=\"\%\$P_fs_$oname\$P_parg*\$oold\$P_parg*%\""
				if [ "x$onew" != "x" ]; then
					if [ "$fold" = "yes" ]; then
						if [ "x$oold" != "x$onew" ]; then
							case "$oname" in
								Summary|Requires)
									P_pre2="$P_pre_Headers_Directory"
									P_post2="$P_post_Headers_Directory"
									eval "com=\"\${p_com}s%^([[:space:]]*)\$P_f_$oname\$P_parg*.*$%\1\\# $oname: \$onew%\""
									;;
								Include)
									eval "com=\"\${p_com}s%^([[:space:]]*)\$P_f_$oname\$P_parg*.*$%\1$oname \"\$onew\"%\""
									;;
							esac
						fi
					else
						case "$oname" in
							Summary|Requires)
								if [ "$fheader" = "yes" ]; then
									P_pre2="$P_pre_Headers_Directory"
									P_post2="$P_post_Headers_Directory"
									eval "com=\"1,1 i\\\\\t\\t\\# \$oname: \$onew\""
								else
									eval "com=\"1,1 a\\\\\t\\t\\# \$oname: \$onew\""
								fi
								;;
							Include)
								eval "com=\"/\$P_ZT_Directory/ i\\\\\t\\t\$oname \"\$onew\"
\""
								;;
						esac
					fi
				else
					com="${p_com} d"
				fi

				sed -i -r "$P_pre
$P_pre2
$com
$P_post2
$P_post" \
						"$fname"
			fi
		}
}

